==> moloco-sdk-3.3.0-sources.jar: com/moloco/sdk/xenoss/sdkdevkit/android/adrenderer/internal/VastBanner.kt <==
package com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal

import android.annotation.SuppressLint
import android.content.Context
import com.moloco.sdk.internal.ortb.model.Bid
import com.moloco.sdk.service_locator.SdkObjectFactory
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.*
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.VastAdLoader
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.render.ad.AdController
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.render.ad.vastAdPlaylistController
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.render.ad.AdControllerEvent
import com.moloco.sdk.xenoss.sdkdevkit.android.core.services.CustomUserEventBuilderService
import kotlinx.coroutines.flow.*

/**
 * @param options mostly for changing renderer behaviour, like button customization, when to show close button, mute etc. Same as for [FullscreenAd.show]
 */
internal fun VastBanner(
    context: Context,
    customUserEventBuilderService: CustomUserEventBuilderService,
    bid: Bid,
    options: VastOptions,
    externalLinkHandler: ExternalLinkHandler,
    vastMediaStreamingEnabled: Boolean,
    loadVast: VastAdLoader = VastAdLoader(context),
    decLoader: DECLoader = SdkObjectFactory.AdLoadModule.decLoader,
): Banner<VastAdShowListener> = VastBannerImpl(
    context,
    customUserEventBuilderService,
    bid,
    options,
    loadVast,
    decLoader,
    externalLinkHandler,
    vastMediaStreamingEnabled
)

@SuppressLint("ViewConstructor")
private class VastBannerImpl(
    private val context: Context,
    private val customUserEventBuilderService: CustomUserEventBuilderService,
    bid: Bid,
    private val options: VastOptions,
    loadVast: VastAdLoader,
    decLoader: DECLoader,
    val externalLinkHandler: ExternalLinkHandler,
    private val vastMediaStreamingEnabled: Boolean,
) : Banner<VastAdShowListener>(context) {
    init {
        tag = "MolocoVastBannerView"
    }

    override val creativeType: CreativeType = CreativeType.VAST

    private var adController: AdController? = null

    override fun destroy() {
        super.destroy()
        adController?.destroy()
        adController = null
    }

    override val adLoader = VastAdLoad(bid, scope, loadVast, decLoader, vastMediaStreamingEnabled)

    override fun prepareAdViewForDisplay() {
        val ad = when(val vastAdResult = adLoader.vastAdLoadResult) {
            is Result.Failure -> {
                val error = vastAdResult.value
                adShowListener?.onShowError(error)
                return
            }
            is Result.Success -> {
                vastAdResult.value
            }
        }

        val adController = vastAdPlaylistController(
            ad,
            externalLinkHandler,
            context,
            customUserEventBuilderService,
            mute = options.startMuted,
            overrideLinearGoNextActionEnabled = options.overrideLinearGoNextActionEnabled,
            overrideLinearGoNextActionEnabledDelaySeconds = options.overrideLinearGoNextActionEnabledDelaySeconds,
            companionGoNextActionDelaySeconds = options.companionGoNextActionDelaySeconds,
            decGoNextActionDelaySeconds = options.decGoNextActionDelaySeconds,
            autoStoreOnSkip = options.autoStoreOnSkip,
            autoStoreOnComplete = options.autoStoreOnComplete
        )
        this.adController = adController

        adView = options.VastRenderer(context, adController)

        listenToPlayerEvents()

        adController.show()
    }

    private fun listenToPlayerEvents() {
        adController?.event?.onEach { event ->
            when (event) {
                AdControllerEvent.Skip -> adShowListener?.onVastCompletionStatus(skipped = true)
                AdControllerEvent.Complete -> adShowListener?.onVastCompletionStatus(
                    skipped = false
                )
                AdControllerEvent.ClickThrough -> adShowListener?.onClick()
                is AdControllerEvent.Error -> adShowListener?.onShowError(event.error)
                // I don't care for these events for updating ad displaying state as opposed to VastFullscreenAd.
                AdControllerEvent.LinearDisplayStarted -> { /* no-op */ }
                AdControllerEvent.CompanionDisplayStarted -> { /* no-op */ }
                AdControllerEvent.DECDisplayStarted -> { /* no-op */ }
                AdControllerEvent.Replay -> { /* no-op */ }
                AdControllerEvent.Dismiss -> { /* no-op */ }
            }
        }?.launchIn(scope)
    }
}

==> moloco-sdk-3.6.1-sources.jar: com/moloco/sdk/xenoss/sdkdevkit/android/adrenderer/internal/VastBanner.kt <==
package com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal

import android.annotation.SuppressLint
import android.content.Context
import com.moloco.sdk.internal.Result
import com.moloco.sdk.internal.ortb.model.Bid
import com.moloco.sdk.service_locator.SdkObjectFactory
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.Banner
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.CreativeType
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.FullscreenAd
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.VastAdShowListener
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.VastOptions
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.VastAdLoader
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.render.ad.AdController
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.render.ad.AdControllerEvent
import com.moloco.sdk.xenoss.sdkdevkit.android.adrenderer.internal.vast.render.ad.vastAdPlaylistController
import com.moloco.sdk.xenoss.sdkdevkit.android.core.services.CustomUserEventBuilderService
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach

/**
 * @param options mostly for changing renderer behaviour, like button customization, when to show close button, mute etc. Same as for [FullscreenAd.show]
 */
internal fun VastBanner(
    context: Context,
    customUserEventBuilderService: CustomUserEventBuilderService,
    bid: Bid,
    options: VastOptions,
    externalLinkHandler: ExternalLinkHandler,
    vastMediaStreamingEnabled: Boolean,
    loadVast: VastAdLoader = VastAdLoader(context),
    decLoader: DECLoader = SdkObjectFactory.AdLoadModule.decLoader,
): Banner<VastAdShowListener> = VastBannerImpl(
    context,
    customUserEventBuilderService,
    bid,
    options,
    loadVast,
    decLoader,
    externalLinkHandler,
    vastMediaStreamingEnabled
)

@SuppressLint("ViewConstructor")
private class VastBannerImpl(
    private val context: Context,
    private val customUserEventBuilderService: CustomUserEventBuilderService,
    bid: Bid,
    private val options: VastOptions,
    loadVast: VastAdLoader,
    decLoader: DECLoader,
    val externalLinkHandler: ExternalLinkHandler,
    private val vastMediaStreamingEnabled: Boolean,
) : Banner<VastAdShowListener>(context) {
    init {
        tag = "MolocoVastBannerView"
    }

    override val creativeType: CreativeType = CreativeType.VAST

    private var adController: AdController? = null

    override fun destroy() {
        super.destroy()
        adController?.destroy()
        adController = null
    }

    override val adLoader = VastAdLoad(bid, scope, loadVast, decLoader, vastMediaStreamingEnabled)

    override fun prepareAdViewForDisplay() {
        val ad = when(val vastAdResult = adLoader.vastAdLoadResult) {
            is Result.Failure -> {
                val error = vastAdResult.value
                adShowListener?.onShowError(error)
                return
            }
            is Result.Success -> {
                vastAdResult.value
            }
        }

        val adController = vastAdPlaylistController(
            ad,
            externalLinkHandler,
            context,
            customUserEventBuilderService,
            mute = options.startMuted,
            overrideLinearGoNextActionEnabled = options.overrideLinearGoNextActionEnabled,
            overrideLinearGoNextActionEnabledDelaySeconds = options.overrideLinearGoNextActionEnabledDelaySeconds,
            companionGoNextActionDelaySeconds = options.companionGoNextActionDelaySeconds,
            decGoNextActionDelaySeconds = options.decGoNextActionDelaySeconds,
            autoStoreOnSkip = options.autoStoreOnSkip,
            autoStoreOnComplete = options.autoStoreOnComplete
        )
        this.adController = adController

        adView = options.VastRenderer(context, adController)

        listenToPlayerEvents()

        adController.show()
    }

    private fun listenToPlayerEvents() {
        adController?.event?.onEach { event ->
            when (event) {
                AdControllerEvent.Skip -> adShowListener?.onVastCompletionStatus(skipped = true)
                AdControllerEvent.Complete -> adShowListener?.onVastCompletionStatus(
                    skipped = false
                )
                AdControllerEvent.ClickThrough -> adShowListener?.onClick()
                is AdControllerEvent.Error -> adShowListener?.onShowError(event.error)
                // I don't care for these events for updating ad displaying state as opposed to VastFullscreenAd.
                AdControllerEvent.LinearDisplayStarted -> { /* no-op */ }
                AdControllerEvent.CompanionDisplayStarted -> { /* no-op */ }
                AdControllerEvent.DECDisplayStarted -> { /* no-op */ }
                AdControllerEvent.Replay -> { /* no-op */ }
                AdControllerEvent.Dismiss -> { /* no-op */ }
            }
        }?.launchIn(scope)
    }
}
